clear all
close all
clc

%% Example specific information
mainfolder = cd; % make string with main folder code
addpath(genpath(mainfolder)); % add all subfolders in path

% load data set
load data186_90d.txt  % Filename of measured data 

Nday = 90;              % Length of the dataset in days
idx  = [1:Nday*6*24];   % index amount of data grouped

y90days =  data186_90d(idx,[4 5 6 7]);    % reads the colums with temperature data (minimum three columns)
u90days =  data186_90d(idx,2);            % reads the column with temperature as u=upper model boundary in .txt file


%% Necessary input information

% Temperature measurements in [K]
y = y90days; % at several depths
u = u90days; % preferably channel with best S-N-R, generally, top channel

% relative depths
x =  [0.17 0.20 0.25 0.35]; % [m]

% what to estimate
estimate = 'q';     % estimate both q and k or 'q' (q only)  or 'k' (k only)

% physical parameters
cw_x_rhow = 4.18e6;  % Volumetric heat capacity of water in [J m^-3 K^-1]
c_x_rho = 3.20e6;    % Volumetric heat capacity of saturated soil in [J m^-3 K^-1]

% sampling rate data in [t_u (e.g. s or days)]
dt = 600/(24*60*60); %  s -> days
F_max = floor(1.5*90); % maximum frequency to be considered [1/days]

% Intialization values for alg.
q_in = 1e-10*(24*60*60);  % Intial guess Vertical flow [m/day]
k_in = 1.4*(24*60*60);    % Intial guess effective thermal conductivity [J/(day m K)]

% options calculations (does not need to be specified)
options.dofLPM = 6;
options.maxNbfOfSteps = 100;

%% ++++++++++++ LPMLE3 CODE ++++++++++++++++++++++ %%
help LPMLE3

[q,k,x_c,std_q,std_k,std_qk,freq,Ynt,Ynt_std,U,Val]...
    = LPMLE3(y,u,x,estimate,dt,F_max,c_x_rho,cw_x_rhow,q_in,k_in,options);

%% ++++++++++++ Plot examples  +++++++++++++++++++ %%

set(0,'defaultaxesfontsize',14);
set(0,'defaultlinelinewidth',1.5);

%% Define confidence bound
BND = 0.95; % 95 procent 
CBND = sqrt(2)*erfinv(BND); % central confidence bound Gaussian distribution

%% Plot estimates with confidence bounds
figure(1)
subplot(2,1,1)
title('Estimates')
errorbar(x_c,q,CBND*std_q,'kx')
xlabel('x [m]')
ylabel('Vertical flux [m/d]')
grid on
subplot(2,1,2)
errorbar(x_c,k/86400,CBND*std_k,'kx')
xlabel('x [m]')
ylabel('Thermal conductivity [W/(m K)]')
grid on 

%% Plot SNRs
figure(2)
plot(x,20*log10(abs(Ynt(:,end)/Ynt_std(:,end))),'kx--',...
      x,20*log10(abs(Ynt(:,1)/Ynt_std(:,1))),'kx-',...
     'Markersize',12)    
xlabel('x [m]')
ylabel('Amplitude [dB]')
legend('SNR_{H_1}','SNR_{H_{end}}','location','South','Orientation','horizontal')
grid on
box on

%% Validation
a_min = -20; a_max = 20; agrid = 1000;
c_min =   0; c_max = 50; cgrid = 1000;

[Pbest_loc,PBest_LPMLE3] = Plot_cost_function_validation(a_min,a_max,agrid,c_min,c_max,cgrid,Val);
Error = (PBest_LPMLE3 - Pbest_loc)./PBest_LPMLE3;
