function [q,k,x_c,std_q,std_k,std_qk,freq,Ynt,Ynt_std,U,Val_in] = LPMLE3(y,u,x,estimate,dt,F_max,c_x_rho,cw_x_rhow,q_in,k_in,varargin)
% This function calculates the vertical water flux and/or effective thermal 
% conductivity from temperature measurements measured at at least 
% three vertical temperature locations assuming that it is constant between
% these three locations.
%
% [q,k,x_c,std_q,std_k,std_qk,freq,Ynt,CYnt,U,Pinfo] 
%     = LPMLE3(y,u,x,estimate,dt,F_max,c_x_rho,cw_x_rhow,q_in,k_in,options)
%
%Input to function
%   Temperature measurements
%   y         [K] [time,x]    Temperature measurements used to calculate q
%                               (minimum size three spatial channels)
%   u         [K] [time,x]    Temperature measurement assumed noiseless
%                               (generally top channel)
%   x         [m]             Spatial locations
%
%   'estimate' choose:        'q' or 'k' or 'qk' 
%                             Choose which parameter(s) to be estimated
%   dt        [t_u]           Sample rate measurement (t_u is in days or s)
%   F_max     [1/t_u]         Maximum frequency to be considered
%
%   cw_x_rhow [J m^-3 K^-1]   Volumetric heat capacity of water in 
%   c_x_rho   [J m^-3 K^-1]   Volumetric heat capacity of saturated soil in 
%
%   q_in      [m/t_u]         Intial guess Vertical flow component 
%                             (note in case only 'k' estimated q = q_in)  
%   k_in      [J/(t_u m K)]   Intial guess effective thermal conductivity
%                             (note in case only 'q' estimated k = k_in)
%
%   Optional
%
%   options.dofLPM (default = 6)   Degrees of freedom of local polynomial
%   options.maxNbfOfSteps (default = 100) Number of steps before abortion
%                                       optimaztion algorithm 
%
%Output of function
%   q       [m/t_u]           Vertical flow component 
%   k       [J/(t_u m K)]     Effective thermal conductivity
%   x_c     [m]               Central measurement point of domain
%
%   std_q   [m/t_u]           Standard deviation of vertical flow component      
%   std_k   [J/(t_u m K)]     Standard deviation of eff. thermal conductivity
%   std_qk  [sqrt(J/K)/t_u]   Cross-corr. of vk (large numbers dangerous)
%
%   freq    [1/t_u]          Frequency grid of Ynt, U
%   Ynt, U  [K]              Fourier coefficients without transient
%   Ynt_std [K]              Standard deviation of Fourier coefficients Ynt
%
%   Val_in                   Values of LPMLE3 used for post-processing alg.
%
% Copyright information
%   Copyright (c) 2017 Matthijs van Berkel, Gerd Vandersteen, Uwe Schneidewind
%   All rights reserved.
%   Software can be used freely for non-commercial applications only. 
%   The user of this code is fully responsible for the correctness of any
%   results generated by LPMLE3. 
%       If used in publications please reference 
%   [1] van Berkel, M. et al., 2014. Frequency domain sample maximum likelihood estimation for spatially dependent parameter estimation in PDEs, Automatica 50(8): 2113  2119. DOI: 10.1016/j.automatica.2014.05.027
%   [2] Schneidewind, U. et al., 2016. LPMLE3 - A Novel Method to Quantify Vertical Water Fluxes in Streambeds Using Heat as a Tracer, Water Resources Research
%   Local Polynomial Method is based on Copyright (c) 2012 Rik Pintelon:
%   [3] Pintelon, R. et al., 2010. Estimation of nonparametric noise and FRF models for multivariable systems-Part I: Theory, Mechanical Systems and Signal Processing, 24(3), 573-595.
%
%   Questions: schneidewind@rwth-aachen.de

%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++%
%% ++++++++++++++++++++++++++ Input definement +++++++++++++++++++++++++ %%
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++%

% Check if options have been defined
if ~isempty(varargin)
    options.dofLPM = 6;
    options.maxNbfOfSteps = 100;
else
    if ~isfield(options, 'dofLPM'); options.dofLPM = 6; end
    if ~isfield(options, 'maxNbfOfSteps'); options.maxNbfOfSteps = 100; end
end

% Check size of y and u is correct
if size(y,1) < size(y,2); y = y'; warning('y dimensions are upside down transpose is used'); end
if size(u,1) < size(u,2); u = u'; warning('u dimensions are upside down transpose is used'); end

%% idxFree definement in terms of 'v', 'k', 'vk'
if      strcmpi(estimate,'q'); idxFree = [1]; 
elseif  strcmpi(estimate,'k'); idxFree = [3];
elseif  strcmpi(estimate,'qk'); idxFree = [1,3];
else error('no or wrong parameter to be estimated defined'); 
end

%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++%
%% ++++++++++++++++++++++++++++ Calculations +++++++++++++++++++++++++++ %%
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++%

%% Calculate Fourier transform, separate transient (non-periodic), variance estimate

% Calculate frequency vector
L = size(y,1);
freq = [1:F_max]/(dt*L); % Calculate frequency range

% Calculate fourier transform
Y1      = fft(y);   Y1red = Y1(2:F_max+1,:).';   % remove steady-state value (LPM does not use it)
Ubeg    = fft(u);   U1red = Ubeg(2:F_max+1,:).'; % remove steady-state value (LPM does not use it)

omega   = 2*pi*freq; % define omega

% Make structure of Local Polynomial Method input
data    = struct('Y', Y1red, 'U', U1red, 'freq', freq);
method  = struct('transient', 1, 'dof', options.dofLPM); % transient estimate on, dof degrees of freedom of polynomial

% Apply local polynomial method to calculate Y and its variances
[CY, Y] = LocalPolyAnal(data, method);

%% Reshape signal for use in MLE3 code

ndom    = size(Y.m_nt,1)-2; % every domain consists of three points (hence 3-2 = 1 domain)
nF      = 1:F_max;
x_c     = x((2:length(x)-1));

for i = size(Y.m_nt,1)-2:-1:1;
    xin(i,:)        = [x(i),x(i+1),x(i+2)];
    Tin(:,:,i)      = [Y.m_nt(i,nF);Y.m_nt(i+1,nF);Y.m_nt(i+2,nF)].';
    VarCovar(:,:,i) = [squeeze(CY.m_nt((i+1),(i+1),nF)),squeeze(CY.m_nt((i),(i),nF)),squeeze(CY.m_nt((i+2),(i+2),nF)),...
                       squeeze(CY.m_nt((i),(i+1),nF)),squeeze(CY.m_nt((i+2),(i+1),nF)),squeeze(CY.m_nt((i+2),(i),nF))];
%!!!! Check conjugate of last three columns (Uwe this I need to check, but I don't know the definition in the LP code, yet)
end

%% Calculation of Maximum Likelihood Estimate of transport coefficients %%
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++%
%++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++%
% The definitions in the MLE3 code are:
%   dT/dt = D dT^2/dz^2 + V dT/dz - K T with a  = V/(2D); b  = K/D; c = 1/D;

%% Transform in terms of [a,b,c] for better convergence optimation algorithm

a = -0.5*q_in*cw_x_rhow/k_in; % parameter alpha = (V/2D)
b =  0; % always zero
c = c_x_rho/k_in; % parameter gamma

P0 = [a;b;c];  % make P0 vector
Pstart = P0; % used to intialize algorithm

%% Calculate for every domain the MLE estimate in terms of PBest = [a,b,c] and vk;
for i = ndom:-1:1;
%% Calculate optimal parameters in terms of P
     [PBest, CostBest, CovP] = MLE3Optimization...
         (Pstart,xin(i,:),omega(nF).',Tin(:,:,i),VarCovar(:,:,i),options,idxFree); % Estimates parameters in P
     
%% Transform parameters in terms of P to vk
         [qk, Cqk] = ac2qk(PBest, CovP, c_x_rho, cw_x_rhow); % Extracts the correct flux

%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++%
%% +++++++++++++++++++++++++++++ Store data ++++++++++++++++++++++++++++ %%
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++%      
         q(i) = qk(1); k(i) = qk(2); std_q(i) = sqrt(Cqk(1,1)); std_k(i) = sqrt(Cqk(2,2)); std_qk(i) = sqrt(Cqk(1,2)); % Save main data
    
         CostBestAll(i) = CostBest;  PBestAll(:,i) = PBest; CovPall(:,:,i) = CovP; % store data
end
% Store information in terms of P = [a,c];


% Export Fourier coefficients without transient (without steady-state value)
Ynt = Y.m_nt; U = U1red; 
for i = size(CY.m_nt,3):-1:1; Ynt_std(:,i) = sqrt(diag(CY.m_nt(:,:,i))); end

Val_in.Pstart = Pstart; Val_in.xin = xin; Val_in.omega = omega; Val_in.Tin = Tin;
Val_in.VarCovar = VarCovar; Val_in.options = options; Val_in.nF = nF;
Val_in. CostBestAll = CostBestAll; Val_in. PBestAll = PBestAll; 
Val_in.CovPall = CovPall; Val_in.idxFree = idxFree;
Val_in.ndom = ndom;
end

