function [Pbest_loc,PBest_LPMLE3] = Plot_cost_function_validation(a_min,a_max,agrid,c_min,c_max,cgrid,Val)
%This function plots the cost function for the interval defined *_min *_max
%and plot the cost-function validation, which will be above the upper
%boundary for real measurement data. The output gives the best values in
%alpha and beta for the grid at hand.
%   [Pbest_loc] =
%        Plot_cost_function_validation(a_min,a_max,agrid,c_min,c_max,cgrid,Val)
%
%Input
%   a_min  minimum value for alpha
%   a_max  maximum value for alpha
%   agrid  number of grid points on the domain [a_min,a_max]
%
%   c_min  minimum value for gamma
%   c_max  maximum value for gamma
%   cgrid  number of grid points on the domain [c_min,c_max]
%
%   Val    the values generated by the optimization alg. LPMLE3.m
%
%Output
%   Pbest_loc       optimal value for the grid defined here
%   PBest_LPMLE3    optimal value of the LPMLE3 alg.
%

%% Calculate cost function in a_direction with c = c_best (if fixed c_start)
parameter_rangea = linspace(a_min,a_max,agrid);
for  i = Val.ndom:-1:1;
     Ptest = Val.PBestAll(:,i);
     for j = length(parameter_rangea):-1:1
         Ptest(1) = parameter_rangea(j);
         [Eps,~] = MLE3cost(Ptest,Val.xin(i,:),Val.omega(Val.nF).',Val.Tin(:,:,i),Val.VarCovar(:,:,i));
         Cost_plot1Da(i,j) = real(Eps'*Eps);
     end
     ind_cost_a(i) = find(min(Cost_plot1Da(i,:)) == Cost_plot1Da(i,:));
end

%% Calculate cost function in c_direction with a = a_best (if fixed a_start)
parameter_rangec = linspace(c_min,c_max,cgrid);
for  i = Val.ndom:-1:1;
     Ptest = Val.PBestAll(:,i);
     for j = length(parameter_rangec):-1:1
         Ptest(3) = parameter_rangec(j);
         [Eps,~] = MLE3cost(Ptest,Val.xin(i,:),Val.omega(Val.nF).',Val.Tin(:,:,i),Val.VarCovar(:,:,i));
         Cost_plot1Dc(i,j) = real(Eps'*Eps);
     end
     ind_cost_c(i) = find(min(Cost_plot1Dc(i,:)) == Cost_plot1Dc(i,:));
end


%% Plot cost function


figure
%% over alpha
j = 1:2:21;
for  i = Val.ndom:-1:1;
    subplot(Val.ndom,2,j(i))
    plot(parameter_rangea,Cost_plot1Da(i,:),'k.-',...
             Val.Pstart(1)*ones(10,1),max(Cost_plot1Da(i,:))*linspace(0,1,10),'x',...
             parameter_rangea(ind_cost_a(i)),Cost_plot1Da(i,ind_cost_a(i)),'rd',...
             Val.PBestAll(1,i),Val.CostBestAll(i),'bo')
    ylabel('Cost function (\alpha,\gamma)')
    grid on
    xlabel('\alpha')
    title(sprintf('Domain: x = [%.2f, %.2f, %.2f]',Val.xin(i,1),Val.xin(i,2),Val.xin(i,3)))
end
% legend('Cost','Cost_{best}','initial value','min on plot')    
        
%% over gamma
j = 2:2:20;
for  i = Val.ndom:-1:1;
    subplot(Val.ndom,2,j(i))
    plot(parameter_rangec,Cost_plot1Dc(i,:),'k.-',...
             Val.Pstart(3)*ones(10,1),max(Cost_plot1Dc(i,:))*linspace(0,1,10),'x',...
             parameter_rangec(ind_cost_c(i)),Cost_plot1Dc(i,ind_cost_c(i)),'rd',...
             Val.PBestAll(3,i),Val.CostBestAll(i),'bo')
    ylabel('Cost function (\alpha,\gamma)')
    grid on
    xlabel('\gamma')
    title(sprintf('Domain: x = [%.2f, %.2f, %.2f]',Val.xin(i,1),Val.xin(i,2),Val.xin(i,3)))
end
legend('Cost','initial value','min on plot','LPMLE3')    


%% Export optimal values on map
for  i = Val.ndom:-1:1;
    Pbest_loc(1,i) = parameter_rangea(ind_cost_a(i)); 
    Pbest_loc(3,i) = parameter_rangec(ind_cost_c(i));
end
PBest_LPMLE3 = Val.PBestAll;

%% Cost function validation

BND = 0.95; % 95 procent 
z = sqrt(2)*erfinv(BND); % central confidence bound Gaussian distribution

% Perfect Cost function value Cost_mean = F-n_theta/2;
Vnoise = ones(length(Val.xin(:,2)),1)*(length(Val.nF)-length(Val.idxFree)/2);
Vbnd   = ones(length(Val.xin(:,2)),1)*z*sqrt(Vnoise(1));
V_ML = Val.CostBestAll;


figure
plot(Val.xin(:,2),Vnoise,'k',...
     Val.xin(:,2),Vnoise+Vbnd,'k--',...
     Val.xin(:,2),Vnoise-Vbnd,'k--',...
     Val.xin(:,2),V_ML,'ko-')
legend('Cost',['Conf. bnd = ',num2str(100*BND),'%'])
xlabel('x [m]')
ylabel('V_{ML}')
legend('V_{noise}','95% conf. bnd.') 
grid on
axis 'tight'
title('Cost Function Validation (must be higher than lower bnd)')

end        
