%function [Uout,dUda,dUdb,dUdc] = SlabCostFunction(a,b,c,x,omega,U,VarCovar)

function [Eps,dEpsda,dEpsdb,dEpsdc] = SlabCostFunction(a,b,c,x,omega,U,VarCovar)
% Calculates cost function and its derivatives for slab-geometry
    % [Eps,dEpsda,dEpsdb,dEpsdc] = SlabCostFunction(a,b,c,x,omega,U,VarCovar)
    %  Parameters a b c
    %  x is a vector given by x = [x1,xc,x2]
    %  The temperature measurements are orderd U = [U1,Uc,U2];
    %  The variances are ordered as
    %  VarCovar = [Varyy,Varu1u1,Varu2u2,Covaryu1,Covaryu2,Covaru1u2];
    % dT/dt = D dT^2/dz^2 + V dT/dz - K T with a  = V/(2D); b  = K/D; c =
    % 1/D; with BC: T(x1) = T(x) and T(x3) = T(x)

% Copyright (c) 2015 Matthijs van Berkel, Gerd Vandersteen
% All rights reserved.
% Software can be used freely for non-commercial applications only. 
% If used in publications please reference 
% [1] van Berkel, M., Vandersteen, G., Geerardyn, E., Pintelon, R., Zwart, H.J. and de Baar, M. R., 2014. Frequency domain sample maximum likelihood estimation for spatially dependent parameter estimation in PDEs, Automatica 50(8): 2113  2119. DOI: 10.1016/j.automatica.2014.05.027


%% Model =================================================================

%% mu and position
x1 = x(1); xc = x(2); x2 = x(3); % Measurement positions --x1---xc---x2--
mu = sqrt(a.^2+b+1i.*c.*omega); % mu (lambda =-a/+mu)

%% Denominator Transfer function (noemer)
Denom = exp(2*x1*mu)-exp(2*x2*mu);

%% Transfer functions
G1 = exp(-xc*(a+mu)).*(...
     (exp((a+mu)*x1+2*xc*mu)-exp((a+mu)*x1+2*x2*mu))...
      )./Denom;
  
G2 = exp(-xc*(a+mu)).*(...
     (exp((a+mu)*x2+2*x1*mu)-exp((a+mu)*x2+2*xc*mu))...
      )./Denom;

%% Model (Combined Transfer function)
 Uc = G1(:).*U(:,1) + G2(:).*U(:,3);

%% Jacobian ==============================================================

%% Derivatives of mu
dmuda = a.*(a.^2+b+1i.*c.*omega).^(-1/2);
dmudb = (1/2).*(a.^2+b+1i.*c.*omega).^(-1/2);
dmudc = (1i*(1/2)).*omega.*(a.^2+b+1i.*c.*omega).^(-1/2);

%% Derivatives of G1

%% The derivatives contain a number of common terms
ProdG1 = exp((x1-xc)*(a+mu))./Denom.^2;
CommonTermG1 = (exp(2.*mu*(xc+x1))-exp(4.*mu.*x2))*(xc-x1)...
    +(-exp(2*mu*(xc+x2))+exp(2.*mu.*(x1+x2)))*(xc+x1-2*x2);
    
dG1dbc = ProdG1.*CommonTermG1;


dG1da = ProdG1.*(...
    (exp(2.*xc.*mu)-exp(2.*x2.*mu)).*(exp(2.*x2.*mu)-exp(2.*x1.*mu))*(xc-x1)...
    +dmuda.*CommonTermG1);

dG1db = dmudb.*dG1dbc;

dG1dc = dmudc.*dG1dbc;

%% Derivatives of G2

ProdG2 = exp((x2-xc)*(a+mu))./Denom.^2;

CommonTermG2 = (exp(2.*mu.*(xc+x2))-exp(4.*mu.*x1))*(xc-x2)...
    +(exp(2.*mu.*(x1+x2))-exp(2.*mu.*(xc+x1)))*(xc+x2-2*x1);
    
dG2dbc = ProdG2.*CommonTermG2;


dG2da = ProdG2.*(...
    (exp(2.*xc.*mu)-exp(2.*x1.*mu)).*(exp(2.*x1.*mu)-exp(2.*x2.*mu))*(xc-x2)...
    +dmuda.*CommonTermG2);

dG2db = dmudb.*dG2dbc;

dG2dc = dmudc.*dG2dbc;

%% Derivatives of dUda,dUdb, and dUdc
dUda = dG1da(:).*U(:,1)+dG2da(:).*U(:,3);
dUdb = dG1db(:).*U(:,1)+dG2db(:).*U(:,3);
dUdc = dG1dc(:).*U(:,1)+dG2dc(:).*U(:,3);


%% Uncertainty sigma_e
Var_e = Var_error(G1,G2,VarCovar);
Std_e = sqrt(Var_e);

% Std_e
% abs(U(:,2)-Uc)
%% Derivatives Variance uncertainty
dVarda = dVardtheta(G1,G2,dG1da,dG2da,VarCovar);
dVardb = dVardtheta(G1,G2,dG1db,dG2db,VarCovar);
dVardc = dVardtheta(G1,G2,dG1dc,dG2dc,VarCovar);

%% Calculate Cost function and its derivatives
Eps =  (U(:,2)-Uc)./Std_e;

dEpsda = -dUda./Std_e - 0.5./Var_e.*dVarda.*Eps;
dEpsdb = -dUdb./Std_e - 0.5./Var_e.*dVardb.*Eps; 
dEpsdc = -dUdc./Std_e - 0.5./Var_e.*dVardc.*Eps;


end




