%% This script is intended to demonstrate the capabilities of the LPML 
% method as published by Gerd Vandersteen (Gerd.Vandersteen@vub.ac.be) in
% the paper 
% G. Vandersteen, U.Schneidewind, C.Anibas, C.Schmidt, P.Seuntjens, and O.Batelaan, 
% Determining groundwater-surface water exchange from temperature-time series: 
%    Combining a local polynomial method with a maximum likelihood estimator, 
% Water Resources Research, 51, 922-939, doi:10.1002/2014WR015994, 2015.

%%
% Prepare the data:
%   Loading the data and select the window used for analysis
load ML1_90.txt  % Filename of measured data set
xall = [0 0.15 0.17 0.20 0.25 0.35 0.55]; % Depths of ALL sensors in m
Nday=90;    % Length of the dataset in days 
idx=[1:Nday*6*24]; % index amount of data grouped
u=ML1_90(idx,2);     % Reads the first column as u (=upper model boundary)
idxSensors = 1:6;
y=ML1_90(idx,2+idxSensors); % Read the column as y (= inner temperature measurements)
x=xall(1+idxSensors)-xall(1);   % Depths of the sensors in m 

% Some options
% Use 1 day period only?
USE_ONE_DAY_PERIOD_ONLY = false;

%% Fix the maximum frequency bin F used. 
% The actual maximum frequency equals F/Nday
F=1.5*Nday; 

% Determine the input spectra U, the output spectra Y, and the
% corresponding frequency axis
U=fft(u);U=U(2:F+1,:).';
Y=fft(y);Y=Y(2:F+1,:).';
freq = [1:F]./Nday;

%% Use the Local Polynomial method to determine the transfer functions (transfer function from boundary until the sensor)
data= struct('Y', Y, 'U', U, 'freq', freq);
method=struct('transient', 1, 'dof', 4);
[CY, Y, TY, G_, CvecG_, dof, CL] = LocalPolyAnal(data, method);

%% Use the resulting transfer functions (G), but first reshape the data structure...
G=reshape(G_,[size(G_,1)*size(G_,2), size(G_,3)]);
stdG=zeros(size(G));
for ii=1:size(G,1)
    stdG(ii,:)=sqrt(CvecG_(ii,ii,:)); % The standard deviation on the esimate transfer functions.
end

if (USE_ONE_DAY_PERIOD_ONLY)
    idx = 1*Nday;
else
    idx=find(db(G(ii,:)) > db(stdG(ii,:)));
    for ii=2:size(G,1)
        idx=union(idx, find(db(G(ii,:)) > db(stdG(ii,:))));
    end
end

% Let's take a look at the spectra and the uncertainty
figure(1);clf;
for ii=1:size(G,1)
    subplot(size(G,1),1,ii);
    plot(data.freq(idx),db(G(ii,idx)),'x', data.freq(idx),db(stdG(ii,idx)),'.')
    xlabel('Freq (day^{-1})')
    ylabel('FRF (db)')
    legend({'FRF' 'std(FRF)' })
end


%% Setting the data
% Select the FRF and its standard deviation
Tm=G(:,idx);
stdTm=stdG(:,idx);
omega=2*pi*freq(idx);

% Compute starting values starting from physical parameters  
dT=24*60*60;      % upscaling to scale values in seconds towards days 
q = 1e-10*dT;      % specific discharge vector in m s-1
%k = 1.8*dT;       % Thermal condunctivity of the sediment-fluid matrix in J s^-1 m^-1 K^-1
cw_x_rhow = 4.18e6;  % Volumetric heat capacity of the water in J m^-3 K^-1
c_x_rho = 3.07e6;      % Volumetric heat capacity of the fluid and rock in J m^-3 K^-1
D = 5.86319e-7*dT; % Thermal diffusivity m^2 s^-1

% Transform to the (abstract) parameters [a,b,c]
a = -q/D*cw_x_rhow/c_x_rho;
b = 0;
%c = -c_x_rho/k;
c = -1/D;

P0=[a;b;c];
% Specify which parameters are fixed and which are free (e.g. b=0 is fixed in this case)

FixedC = 0;

if (FixedC)
    idxFree = [1]; % activate if only one parameter should be estimated (here q)
else
    idxFree = [1 3]; % activate if 2 parameters should be estimated simultaneously (here q and D)
end

options.maxNbfOfSteps=100;
% Determine the Maximum Likelihood solution
[P_ML, CostBest, CovP] = MLEOptimization(P0,x,omega,Tm,stdTm,options,idxFree);
% Determine the FRF of the ML solution
[T, dTdb, dTdc] = ComputeTF(P_ML(1),P_ML(2),P_ML(3),x,omega);

figure(2);
subplot(2,1,1);plot(omega/2/pi,db(Tm),'x', omega/2/pi,db(T),'-')
    xlabel('Freq (day^{-1})')
    ylabel('|FRF| (db)')
subplot(2,1,2);plot(omega/2/pi,angle(Tm),'x', omega/2/pi,unwrap(angle(T)),'-')
    xlabel('Freq (day^{-1})')
    ylabel('phase(FRF) (rad)')
    
% Convert parameters [a,c] towards the physical parameters q and k,
% together with the covariance matrix.
a=P_ML(1);c=P_ML(3);
% Then again compute ML parameter q and k
%qk = [c_x_rho/cw_x_rhow*a/c; -c_x_rho/c];

% Then again compute ML parameter q and D
qD = [c_x_rho/cw_x_rhow*a/c; -1/c];

% Compute the covariance matrix of q and k
J = [c_x_rho/cw_x_rhow/c -c_x_rho/cw_x_rhow*a/c^2; ...
    0                  1/c^2];

if (FixedC)
    CqD = J*[CovP(idxFree, idxFree) 0;0 0]*J'; % Eq.(24) - activate if only one parameter should be estimated (here q)
	ExpectedCost=size(Tm,1)*size(Tm,2)-1;
else
    CqD = J*CovP(idxFree, idxFree)*J'; % Eq.(24) - activate if 2 parameters should be estimated simultaneously (here q and D)
    ExpectedCost=size(Tm,1)*size(Tm,2)-1/2;
end

% Display the results
format short e
fprintf('Results\n')
fprintf('q=%e in mm/d, (std=%e)\n',qD(1)*1000,sqrt(CqD(1,1))*1000)
fprintf('D=%e in m2/d, (std=%e)\n',qD(2),sqrt(CqD(2,2)))
fprintf('cost=%e\n',CostBest);
fprintf('expected cost=%e, (std=%e)\n', ExpectedCost, sqrt(ExpectedCost));




